@@ -3,12 +3,13 @@
 ###
 ### oktest.py -- new style test utility
 ###
-### $Release: 0.2.2 $
+### $Release: 0.5.0 $
 ### $Copyright: copyright(c) 2010 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 
-__all__ = ('ok', 'not_ok', 'run', 'dummy_file', 'dummy_dir', 'chdir')
+__all__ = ('ok', 'not_ok', 'run', 'chdir', 'spec', 'using',
+           'dummy_file', 'dummy_dir', 'dummy_values', 'dummy_attrs', 'dummy_environ_vars')
 
 import sys, os, re, types, traceback
 
@@ -21,6 +22,11 @@ if python2:
         return isinstance(obj, (types.TypeType, types.ClassType))
     def _func_firstlineno(func):
         return func.im_func.func_code.co_firstlineno
+    def _read_file(fname):
+        f = open(fname)
+        s = f.read()
+        f.close()
+        return s
 if python3:
     def _is_string(val):
         return isinstance(val, (str, bytes))
@@ -28,106 +34,212 @@ if python3:
         return isinstance(obj, (type, ))
     def _func_firstlineno(func):
         return func.__code__.co_firstlineno
+    def _read_file(fname, encoding='utf-8'):
+        #with open(fname, encoding=encoding) as f:
+        #    s = f.read()
+        f = open(fname, encoding=encoding)
+        s = f.read()
+        f.close()
+        return s
+
+def _get_location(depth=0):
+    frame = sys._getframe(depth+1)
+    return (frame.f_code.co_filename, frame.f_lineno)
 
 
-class TestFailed(AssertionError):
+__unittest = True    # see unittest.TestResult._is_relevant_tb_level()
 
-    def __init__(self, mesg, file=None, line=None):
-        AssertionError.__init__(self, mesg)
-        self.file = file
-        self.line = line
+## not used for compatibility with unittest
+#class TestFailed(AssertionError):
+#
+#    def __init__(self, mesg, file=None, line=None, diff=None):
+#        AssertionError.__init__(self, mesg)
+#        self.file = file
+#        self.line = line
+#        self.diff = diff
+#
+
+
+def ex2msg(ex):
+    #return ex.message   # deprecated since Python 2.6
+    #return str(ex)      # may be empty
+    #return ex.args[0]   # ex.args may be empty (ex. AssertionError)
+    #return (ex.args or ['(no error message)'])[0]
+    return str(ex) or '(no error message)'
 
 
 def _msg(target, op, other=None):
     if   op.endswith('()'):   msg = '%r%s'     % (target, op)
     elif op.startswith('.'):  msg = '%r%s(%r)' % (target, op, other)
     else:                     msg = '%r %s %r' % (target, op, other)
+    if op == '==' and target != other and _is_string(target) and _is_string(other):
+        if DIFF:
+            if python2 or isinstance(target, str) and isinstance(other, str):
+                diff = _diff(target, other)
+                return (msg, diff)
     return msg
 
 
+DIFF = True
+
+def _diff(target, other):
+    from difflib import unified_diff
+    if hasattr(DIFF, '__call__'):
+        expected = [ DIFF(line) + "\n" for line in other.splitlines(True) ]
+        actual   = [ DIFF(line) + "\n" for line in target.splitlines(True) ]
+    else:
+        if other.find("\n") == -1 and target.find("\n") == -1:
+            expected, actual = [other + "\n"], [target + "\n"]
+        else:
+            expected, actual = other.splitlines(True), target.splitlines(True)
+            if not expected: expected.append('')
+            if not actual:   actual.append('')
+            for lines in (expected, actual):
+                if not lines[-1].endswith("\n"):
+                    lines[-1] += "\n\\ No newline at end of string\n"
+    return ''.join(unified_diff(expected, actual, 'expected', 'actual', n=2))
+
+
+def assertion_op(func):
+    def deco(self, *args):
+        self._tested = True
+        return func(self, *args)
+    if python2:
+        deco.func_name = func.func_name
+    deco.__name__ = func.__name__
+    deco.__doc__ = func.__doc__
+    return deco
+
+
 #def deprecated(f):
 #    return f
 
 
-class ValueObject(object):
+class AssertionObject(object):
 
-    def __init__(self, value):
+    def __init__(self, value, _bool=True):
         self.value = value
-        self._bool = True
+        self._bool = _bool
+        self._tested = False
+        self._location = None
+
+    def __del__(self):
+        if self._tested is False:
+            msg = "%s() is called but not tested." % (self._bool and 'ok' or 'not_ok')
+            if self._location:
+                msg += " (file '%s', line %s)" % self._location
+            #import warnings; warnings.warn(msg)
+            sys.stderr.write("*** warning: oktest: %s\n" % msg)
 
     #def not_(self):
     #    self._bool = not self._bool
     #    return self
 
     def _failed(self, msg, postfix=' : failed.', depth=2):
-        frame = sys._getframe(depth)
-        file = frame.f_code.co_filename
-        line = frame.f_lineno
+        file, line = _get_location(depth + 1)
+        diff = None
+        if isinstance(msg, tuple):
+            msg, diff = msg
         if self._bool == False:
             msg = 'not ' + msg
         if postfix:
             msg += postfix
-        raise TestFailed(msg, file=file, line=line)
+        raise self._assertion_error(msg, file, line, diff)
 
+    def _assertion_error(self, msg, file, line, diff):
+        #return TestFailed(msg, file=file, line=line, diff=diff)
+        ex = AssertionError(msg)
+        ex.file = file;  ex.line = line;  ex.diff = diff
+        return ex
+
+    @assertion_op
     def __eq__(self, other):
         if (self.value == other) == self._bool:  return True
         self._failed(_msg(self.value, '==', other))
 
+    @assertion_op
     def __ne__(self, other):
         if (self.value != other) == self._bool:  return True
         self._failed(_msg(self.value, '!=', other))
 
+    @assertion_op
     def __gt__(self, other):
         if (self.value > other) == self._bool:  return True
         self._failed(_msg(self.value, '>', other))
 
+    @assertion_op
     def __ge__(self, other):
         if (self.value >= other) == self._bool:  return True
         self._failed(_msg(self.value, '>=', other))
 
+    @assertion_op
     def __lt__(self, other):
         if (self.value < other) == self._bool:  return True
         self._failed(_msg(self.value, '<', other))
 
+    @assertion_op
     def __le__(self, other):
         if (self.value <= other) == self._bool:  return True
         self._failed(_msg(self.value, '<=', other))
 
+    @assertion_op
+    def in_delta(self, other, delta):
+        if (self.value <= other - delta) == self._bool:
+            self._failed(_msg(self.value, '>', other - delta))
+        if (self.value >= other + delta) == self._bool:
+            self._failed(_msg(self.value, '<', other + delta))
+        return True
+
+#    @assertion_op
 #    def __contains__(self, other):
 #        if (self.value in other) == self._bool:  return True
 #        self._failed(_msg(self.value, 'in', other))
+    @assertion_op
     def in_(self, other):
         if (self.value in other) == self._bool:  return True
         self._failed(_msg(self.value, 'in', other))
 
+    @assertion_op
     def not_in(self, other):  # DEPRECATED
         if (self.value not in other) == self._bool:  return True
         self._failed(_msg(self.value, 'not in', other))
 
+    @assertion_op
     def contains(self, other):
         if (other in self.value) == self._bool:  return True
         self._failed(_msg(other, 'in', self.value))
 
+    @assertion_op
     def not_contain(self, other):  # DEPRECATED
         if (other in self.value) == self._bool:  return True
         self._failed(_msg(other, 'not in', self.value))
 
+    @assertion_op
     def is_(self, other):
         if (self.value is other) == self._bool:  return True
         self._failed(_msg(self.value, 'is', other))
 
+    @assertion_op
     def is_not(self, other):
         if (self.value is not other) == self._bool:  return True
         self._failed(_msg(self.value, 'is not', other))
 
+    @assertion_op
     def is_a(self, other):
         if (isinstance(self.value, other)) == self._bool:  return True
         self._failed("isinstance(%r, %s)" % (self.value, other.__name__))
 
+    @assertion_op
     def is_not_a(self, other):  # DEPRECATED
         if (not isinstance(self.value, other)) == self._bool:  return True
         self._failed("not isinstance(%r, %s)" % (self.value, other.__name__))
 
+    @assertion_op
+    def hasattr(self, name):
+        if hasattr(self.value, name) == self._bool:  return True
+        self._failed("hasattr(%r, %r)" % (self.value, name))
+
+    @assertion_op
     def matches(self, pattern):
         if isinstance(pattern, type(re.compile('x'))):
             if bool(pattern.search(self.value)) == self._bool:  return True
@@ -136,6 +248,7 @@ class ValueObject(object):
             if bool(re.search(pattern, self.value)) == self._bool:  return True
             self._failed("re.search(%r, %r)" % (pattern, self.value))
 
+    @assertion_op
     def not_match(self, pattern):  # DEPRECATED
         if isinstance(pattern, type(re.compile('x'))):
             if (not pattern.search(self.value)) == self._bool:  return True
@@ -144,33 +257,41 @@ class ValueObject(object):
             if (not re.search(pattern, self.value)) == self._bool:  return True
             self._failed("not re.search(%r, %r)" % (pattern, self.value))
 
+    @assertion_op
     def is_file(self):
         if (os.path.isfile(self.value)) == self._bool:  return True
         self._failed('os.path.isfile(%r)' % self.value)
 
+    @assertion_op
     def is_not_file(self):  # DEPRECATED
         if (not os.path.isfile(self.value)) == self._bool:  return True
         self._failed('not os.path.isfile(%r)' % self.value)
 
+    @assertion_op
     def is_dir(self):
         if (os.path.isdir(self.value)) == self._bool:  return True
         self._failed('os.path.isdir(%r)' % self.value)
 
+    @assertion_op
     def is_not_dir(self):  # DEPRECATED
         if (not os.path.isdir(self.value)) == self._bool:  return True
         self._failed('not os.path.isdir(%r)' % self.value)
 
+    @assertion_op
     def exists(self):
         if (os.path.exists(self.value)) == self._bool:  return True
         self._failed('os.path.exists(%r)' % self.value)
 
+    @assertion_op
     def not_exist(self):  # DEPRECATED
         if (not os.path.exists(self.value)) == self._bool:  return True
         self._failed('not os.path.exists(%r)' % self.value)
 
+    @assertion_op
     def raises(self, exception_class, errmsg=None):
         return self._raise_or_not(exception_class, errmsg, self._bool)
 
+    @assertion_op
     def not_raise(self, exception_class=Exception):  # DEPRECATED
         return self._raise_or_not(exception_class, None, not self._bool)
 
@@ -184,10 +305,10 @@ class ValueObject(object):
                 if not isinstance(ex, exception_class):
                     self._failed('%s%r is kind of %s' % (ex.__class__.__name__, ex.args, exception_class.__name__), depth=3)
                     #raise
-                if errmsg is None or str(ex) == errmsg:
+                if errmsg is None or str(ex) == errmsg:  # don't use ex2msg(ex)!
                     return
-                #self._failed("expected %r but got %r" % (errmsg, ex.message))
-                self._failed("%r == %r" % (str(ex), errmsg), depth=3)
+                #self._failed("expected %r but got %r" % (errmsg, str(ex)))
+                self._failed("%r == %r" % (str(ex), errmsg), depth=3)   # don't use ex2msg(ex)!
             self._failed('%s should be raised' % exception_class.__name__, depth=3)
         else:
             try:
@@ -199,19 +320,21 @@ class ValueObject(object):
                     self._failed('%s should not be raised' % exception_class.__name__, depth=3)
 
 
-VALUE_OBJECT = ValueObject
+ASSERTION_OBJECT = AssertionObject
 
 
 def ok(value):
-    return VALUE_OBJECT(value)
+    obj = ASSERTION_OBJECT(value, True)
+    obj._location = _get_location(1)
+    return obj
 
 def not_ok(value):
-    v = ok(value)
-    v._bool = False
-    return v
+    obj = ASSERTION_OBJECT(value, False)
+    obj._location = _get_location(1)
+    return obj
 
 
-class TestClassRunner(object):
+class TestRunner(object):
 
     def __init__(self, klass, reporter=None):
         self.klass = klass
@@ -251,9 +374,10 @@ class TestClassRunner(object):
             obj.__name__ = _test_name(method_name)
             obj._testMethodName = method_name    # unittest.TestCase compatible
             obj._testMethodDoc = func.__doc__    # unittest.TestCase compatible
-            ## invoke before_each() or setUp()
-            reporter.before_each(obj)
-            if   hasattr(obj, 'before_each'):  obj.before_each()
+            ## invoke before() or setUp()
+            reporter.before(obj)
+            if   hasattr(obj, 'before'):       obj.before()
+            elif hasattr(obj, 'before_each'):  obj.before_each()  # for backward compatibility
             elif hasattr(obj, 'setUp'):        obj.setUp()
             try:
                 try:
@@ -269,10 +393,11 @@ class TestClassRunner(object):
                     count += 1
                     reporter.print_error(obj, ex)
             finally:
-                ## invoke before_each() or tearDown()
-                if   hasattr(obj, 'after_each'):  obj.after_each()
+                ## invoke after() or tearDown()
+                if   hasattr(obj, 'after'):       obj.after()
+                elif hasattr(obj, 'after_each'):  obj.after_each()  # for backward compatibility
                 elif hasattr(obj, 'tearDown'):    obj.tearDown()
-                reporter.after_each(obj)
+                reporter.after(obj)
         ## invoke after_all()
         if hasattr(klass, 'after_all'):
             klass.after_all()
@@ -280,27 +405,32 @@ class TestClassRunner(object):
         return count
 
 
-TEST_CLASS_RUNNER = TestClassRunner
+TEST_RUNNER = TestRunner
 
 
-def run(*classes):
-    class_list = []
-    pat_type = type(re.compile('x'))
+TARGET_PATTERN = '.*Test(Case)?$'
+
+def run(*targets):
+    if len(targets) == 0:
+        targets = (TARGET_PATTERN, )
+    target_list = []
+    rexp_type = type(re.compile('x'))
     vars = None
-    for arg in classes:
-        if _is_string(arg) or isinstance(arg, pat_type):
-            pattern = isinstance(arg, pat_type) and arg or re.compile(arg)
-            if vars is None: vars = sys._getframe(1).f_locals
-            class_list.extend( vars[k] for k in vars if pattern.match(k) )
-        elif _is_class(arg):
+    for arg in targets:
+        if _is_class(arg):
             klass = arg
-            class_list.append(klass)
+            target_list.append(klass)
+        elif _is_string(arg) or isinstance(arg, rexp_type):
+            rexp = _is_string(arg) and re.compile(arg) or arg
+            if vars is None: vars = sys._getframe(1).f_locals
+            klasses = [ vars[k] for k in vars if rexp.search(k) and _is_class(vars[k]) ]
+            target_list.extend(klasses)
         else:
             raise Exception("%r: not a class nor pattern string." % arg)
     #
     count = 0
-    for klass in class_list:
-        runner = TEST_CLASS_RUNNER(klass, REPORTER())
+    for klass in target_list:
+        runner = TEST_RUNNER(klass, REPORTER())
         count += runner.run()
     return count
 
@@ -320,10 +450,10 @@ class Reporter(object):
     def after_all(self, klass):
         pass
 
-    def before_each(self, obj):
+    def before(self, obj):
         pass
 
-    def after_each(self, obj):
+    def after(self, obj):
         pass
 
     def print_ok(self, obj):
@@ -348,22 +478,33 @@ class BaseReporter(Reporter):
         if not hasattr(self, '_lines'):
             self._lines = {}
         if file not in self._lines:
-            f = open(file)
-            s = f.read()
-            f.close()
+            if not os.path.isfile(file): return None
+            s = _read_file(file)
             self._lines[file] = s.splitlines(True)
         return self._lines[file][line-1]
 
     def _get_location(self, ex):
         if hasattr(ex, 'file') and hasattr(ex, 'line'):
-            text = self._get_line_text(ex.file, ex.line).strip()
-            return (ex.file, ex.line, text)
+            text = self._get_line_text(ex.file, ex.line)
+            if text: text = text.strip()
+            return (ex.file, ex.line, None, text)
         else:
             tb = traceback.extract_tb(sys.exc_info()[2])
             for file, line, func, text in tb:
                 if os.path.basename(file) not in ('oktest.py', 'oktest.pyc'):
-                    return (file, line, text)
-            return (None, None, None)
+                    return (file, line, func, text)
+            return (None, None, None, None)
+
+    def _write(self, str):
+        OUT.write(str)
+
+    def _write_tb(self, filename, linenum, funcname, linetext):
+        if funcname:
+            self._write('  File "%s", line %s, in %s\n' % (filename, linenum, funcname))
+        else:
+            self._write('  File "%s", line %s\n' % (filename, linenum))
+        if linetext:
+            self._write('    %s\n' % linetext)
 
 
 ## NOTICE! reporter spec will be changed frequently
@@ -381,27 +522,33 @@ class SimpleReporter(BaseReporter):
     def print_ok(self, obj):
         OUT.write("."); OUT.flush()
 
+    def _write(self, str):
+        self.buf.append(str)
+
     def print_failed(self, obj, ex):
         OUT.write("f"); OUT.flush()
-        self.buf.append("Failed: %s()\n" % self._test_ident(obj))
-        self.buf.append("   %s\n" % ex.args[0])
-        file, line, text = self._get_location(ex)
+        self._write("Failed: %s()\n" % self._test_ident(obj))
+        self._write("  %s\n" % ex2msg(ex))
+        file, line, func, text = self._get_location(ex)
         if file:
-            self.buf.append("   %s:%s:  %s\n" % (file, line, text))
+            #self._write("   %s:%s:  %s\n" % (file, line, text))
+            self._write_tb(file, line, func, text)
+        if getattr(ex, 'diff', None):
+            self._write(ex.diff)
 
     def print_error(self, obj, ex):
         OUT.write('E'); OUT.flush()
-        self.buf.append("ERROR: %s()\n" % self._test_ident(obj))
-        self.buf.append("  %s: %s\n" % (ex.__class__.__name__, str(ex)))
+        self._write("ERROR: %s()\n" % self._test_ident(obj))
+        self._write("  %s: %s\n" % (ex.__class__.__name__, ex2msg(ex)))
         #traceback.print_exc(file=sys.stdout)
         tb = traceback.extract_tb(sys.exc_info()[2])
         iter = tb.__iter__()
         for filename, linenum, funcname, linetext in iter:
             if os.path.basename(filename) not in ('oktest.py', 'oktest.pyc'):
                 break
-        self.buf.append(    "  - %s:%s:  %s\n" % (filename, linenum, linetext))
+        self._write_tb(filename, linenum, funcname, linetext)
         for filename, linenum, funcname, linetext in iter:
-            self.buf.append("  - %s:%s:  %s\n" % (filename, linenum, linetext))
+            self._write_tb(filename, linenum, funcname, linetext)
         tb = iter = None
 
 
@@ -417,20 +564,22 @@ class OldStyleReporter(BaseReporter):
     def _test_ident(self, obj):
         return '%s.%s' % (self.klass.__name__, obj._testMethodName)
 
-    def before_each(self, obj):
+    def before(self, obj):
         OUT.write("* %s ... " % self._test_ident(obj))
 
     def print_ok(self, obj):
         OUT.write("[ok]\n")
 
     def print_failed(self, obj, ex):
-        OUT.write("[NG] %s\n" % str(ex))
-        file, line, text = self._get_location(ex)
+        OUT.write("[NG] %s\n" % ex2msg(ex))
+        file, line, func, text = self._get_location(ex)
         if file:
             OUT.write("   %s:%s: %s\n" % (file, line, text))
+        if getattr(ex, 'diff', None):
+            OUT.write(ex.diff)
 
     def print_error(self, obj, ex):
-        OUT.write("[ERROR] %s: %s\n" % (ex.__class__.__name__, str(ex)))
+        OUT.write("[ERROR] %s: %s\n" % (ex.__class__.__name__, ex2msg(ex)))
         #traceback.print_exc(file=sys.stdout)
         tb = traceback.extract_tb(sys.exc_info()[2])
         iter = tb.__iter__()
@@ -446,6 +595,8 @@ class OldStyleReporter(BaseReporter):
 ## NOTICE! reporter spec will be changed frequently
 class TapStyleReporter(BaseReporter):
 
+    BOL_PATTERN = re.compile(r'^', re.M)
+
     def before_all(self, klass):
         self.klass = klass
         OUT.write("### %s\n" % klass.__name__)
@@ -458,14 +609,16 @@ class TapStyleReporter(BaseReporter):
 
     def print_failed(self, obj, ex):
         OUT.write("not ok # %s\n" % self._test_ident(obj))
-        OUT.write("   #  %s\n" % ex.args[0])
-        file, line, text = self._get_location(ex)
+        OUT.write("   #  %s\n" % ex2msg(ex))
+        file, line, func, text = self._get_location(ex)
         if file:
             OUT.write("   #  %s:%s:  %s\n" % (file, line, text))
+        if getattr(ex, 'diff', None):
+            OUT.write(re.sub(self.BOL_PATTERN, '   #', ex.diff))
 
     def print_error(self, obj, ex):
         OUT.write("ERROR  # %s\n" % self._test_ident(obj))
-        OUT.write("   #  %s: %s\n" % (ex.__class__.__name__, str(ex)))
+        OUT.write("   #  %s: %s\n" % (ex.__class__.__name__, ex2msg(ex)))
         #traceback.print_exc(file=sys.stdout)
         tb = traceback.extract_tb(sys.exc_info()[2])
         iter = tb.__iter__()
@@ -497,7 +650,7 @@ class _Context(object):
         return self
 
     def __exit__(self, *args):
-        return self
+        return None
 
     def __call__(self, func, *args):
         self.__enter__()
@@ -541,6 +694,33 @@ class DummyDir(_Context):
         shutil.rmtree(self.path)
 
 
+class DummyValues(_Context):
+
+    def __init__(self, dictionary, items_=None, **kwargs):
+        self.dict = dictionary
+        self.items = {}
+        if isinstance(items_, dict):
+            self.items.update(items_)
+        if kwargs:
+            self.items.update(kwargs)
+
+    def __enter__(self):
+        self.original = d = {}
+        for k in self.items:
+            if k in self.dict:
+                d[k] = self.dict[k]
+        self.dict.update(self.items)
+        return self
+
+    def __exit__(self, *args):
+        for k in self.items:
+            if k in self.original:
+                self.dict[k] = self.original[k]
+            else:
+                del self.dict[k]
+        self.__dict__.clear()
+
+
 class Chdir(_Context):
 
     def __init__(self, dirname):
@@ -556,38 +736,80 @@ class Chdir(_Context):
         os.chdir(self.back_to)
 
 
+class Spec(_Context):
+
+    def __init__(self, desc):
+        self.desc = desc
+
+
+class Using(_Context):
+    """ex.
+         class MyTest(object):
+            pass
+         with oktest.Using(MyTest):
+            def test_1(self):
+              ok (1+1) == 2
+         if __name__ == '__main__':
+            oktest.run(MyTest)
+    """
+    def __init__(self, klass):
+        self.klass = klass
+
+    def __enter__(self):
+        self.locals = sys._getframe(1).f_locals
+        self.start_names = self.locals.keys()
+        if python3: self.start_names = list(self.start_names)
+        return self
+
+    def __exit__(self, *args):
+        curr_names = self.locals.keys()
+        diff_names = list(set(curr_names) - set(self.start_names))
+        for name in diff_names:
+            setattr(self.klass, name, self.locals[name])
+
+
 def dummy_file(filename, content):
     return DummyFile(filename, content)
 
 def dummy_dir(dirname):
     return DummyDir(dirname)
 
+def dummy_values(dictionary, items_=None, **kwargs):
+    return DummyValues(dictionary, items_, **kwargs)
+
+def dummy_attrs(object, items_=None, **kwargs):
+    return DummyValues(object.__dict__, items_, **kwargs)
+
+def dummy_environ_vars(**kwargs):
+    return DummyValues(os.environ, **kwargs)
+
 def chdir(path):
     return Chdir(path)
 
+def spec(desc):
+    return Spec(desc)
 
-if __name__ == '__main__':
+def using(klass):
+    return Using(klass)
 
-    class MyTest(object):
-        def before_each(self):
-            self.L = [10, 20, 30]
-        def test_ex1(self):
-            ok (len(self.L)) == 3
-            ok (sum(self.L)) == 60
-        def test_ex2(self):
-            ok (self.L[-1]) > 31
 
-    run(MyTest)
+##
+## undocumented
+##
 
-    import unittest
-    class MyTestCase(unittest.TestCase):
-        def setUp(self):
-            self.L = [10, 20, 30]
-        def test_ex1(self):
-            ok (len(self.L)) == 3
-            ok (sum(self.L)) == 60
-        def test_ex2(self):
-            ok (self.L[-1]) > 31
+def flatten(arr, type=(list, tuple)):
+    L = []
+    for x in arr:
+        if isinstance(x, type):
+            L.extend(flatten(x))
+        else:
+            L.append(x)
+    return L
 
-    #unittest.main()
-    run(MyTestCase)
+def rm_rf(*fnames):
+    for fname in flatten(fnames):
+        if os.path.isfile(fname):
+            os.unlink(fname)
+        elif os.path.isdir(fname):
+            from shutil import rmtree
+            rmtree(fname)
