@@ -1,5 +1,5 @@
 ###
-### $Release: 0.4.0 $
+### $Release: 0.5.0 $
 ### $Copyright: copyright(c) 2010 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -33,6 +33,10 @@ def do_test_with(desc, script, expected,  _pat=re.compile(r'0\.00[01]s')):
         echo("- %s..." % desc)
         open(filename, 'w').write(script)
         output = os.popen(python_command + ' ' + filename).read()
+        #sin, sout, serr = os.popen(python_command + ' ' + filename)
+        #sin.close()
+        #output = sout.read();  sout.close()
+        #output_err = serr.read();  serr.close()
         if isinstance(output, str):
             output = re.sub(r' at 0x[0-9a-f]{6}', '', output)
         if output == expected:
@@ -445,6 +449,25 @@ expected = r"""
 """[1:]
 do_test_with(desc, script, expected)
 
+###
+desc = "op 'hasattr'"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_hasattr(self):
+        ok ("s").hasattr("__class__")
+class BarTest(object):
+    def test_hasattr(self):
+        ok ("s").hasattr("xxxxx")
+run('FooTest', 'BarTest')
+"""[1:]
+expected = r"""
+* FooTest.test_hasattr ... [ok]
+* BarTest.test_hasattr ... [NG] hasattr('s', 'xxxxx') : failed.
+   _test_.py:7: ok ("s").hasattr("xxxxx")
+"""[1:]
+do_test_with(desc, script, expected)
+
 ###
 desc = "op 'raises'"
 script = r"""
@@ -594,6 +617,68 @@ do_test_with(desc, script, expected)
 
 
 
+### run (with class objects)
+desc = "run (with class objects)"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_1(self):
+        ok (1+1) == 2
+class BarTest(object):
+    def test_2(self):
+        ok (1+1) == 2
+run(FooTest, BarTest)
+"""
+expected = r"""
+* FooTest.test_1 ... [ok]
+* BarTest.test_2 ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+### run (with pattern)
+desc = "run (with pattern)"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_1(self):
+        ok (1+1) == 2
+class BarTestCase(object):
+    def test_2(self):
+        ok (1+1) == 2
+class BazTestCase(object):
+    def test_3(self):
+        ok (1+1) == 2
+run('.*TestCase$')
+"""
+expected = r"""
+* BarTestCase.test_2 ... [ok]
+* BazTestCase.test_3 ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+### run (without args)
+desc = "run (without args)"
+script = r"""
+from oktest import *
+class FooTest(object):
+    def test_1(self):
+        ok (1+1) == 2
+class BarTestCase(object):
+    def test_2(self):
+        ok (1+1) == 2
+class BazTestCase(object):
+    def test_3(self):
+        ok (1+1) == 2
+run()
+"""
+expected = r"""
+* BarTestCase.test_2 ... [ok]
+* BazTestCase.test_3 ... [ok]
+* FooTest.test_1 ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+
 
 ### dummy_file (with with-statement)
 desc = "dummy_file (with with-statement)"
@@ -663,6 +748,141 @@ run(FooTest)
 expected = "* FooTest.test_dummy_dir2 ... [ok]\n"
 do_test_with(desc, script, expected)
 
+### dummy_environ_vars (with with-statement)
+desc = "dummy_environ_vars (with with-statement)"
+script = r"""
+from __future__ import with_statement
+import os
+from oktest import *
+class FooTest(object):
+    def test_dummy_environ_vars(self):
+        os.environ['AAAA'] = 'hoge'
+        os.environ['CCCC'] = ''
+        with dummy_environ_vars(AAAA='aaa', BBBB='bbb', CCCC='ccc'):
+            ok (os.environ['AAAA']) == 'aaa'
+            ok (os.environ['BBBB']) == 'bbb'
+            ok (os.environ['CCCC']) == 'ccc'
+        ok (os.environ.get('AAAA')) == 'hoge'
+        ok ('BBBB' not in os.environ) == True
+        ok ('CCCC' in os.environ) == True
+        ok (os.environ.get('CCCC')) == ''
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_environ_vars ... [ok]\n"
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+### dummy_environ_vars (without with-statement)
+desc = "dummy_environ_vars (without with-statement)"
+script = r"""
+import os
+from oktest import *
+class FooTest(object):
+    def test_dummy_environ_vars2(self):
+        os.environ['AAAA'] = 'hoge'
+        os.environ['CCCC'] = ''
+        def f():
+            ok (os.environ['AAAA']) == 'aaa'
+            ok (os.environ['BBBB']) == 'bbb'
+            ok (os.environ['CCCC']) == 'ccc'
+        dummy_environ_vars(AAAA='aaa', BBBB='bbb', CCCC='ccc')(f)
+        ok (os.environ.get('AAAA')) == 'hoge'
+        ok ('BBBB' not in os.environ) == True
+        ok ('CCCC' in os.environ) == True
+        ok (os.environ.get('CCCC')) == ''
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_environ_vars2 ... [ok]\n"
+do_test_with(desc, script, expected)
+
+### dummy_values (with with-statement)
+desc = "dummy_values (with with-statement)"
+script = r"""
+from __future__ import with_statement
+import os
+from oktest import *
+class FooTest(object):
+    def test_dummy_values(self):
+        d = {'A': 10, 'B': 20, 'C': 30 }
+        with dummy_values(d, {999: 0}, A=100, B=200):
+            ok (d) == {'A': 100, 'B': 200, 'C': 30, 999: 0}
+        ok (d) == {'A': 10, 'B': 20, 'C': 30 }
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_values ... [ok]\n"
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+### dummy_values (without with-statement)
+desc = "dummy_values (without with-statement)"
+script = r"""
+import os
+from oktest import *
+class FooTest(object):
+    def test_dummy_values2(self):
+        d = {'A': 10, 'B': 20, 'C': 30 }
+        def f():
+            ok (d) == {'A': 100, 'B': 200, 'C': 30, 999: 0}
+        dummy_values(d, {999: 0}, A=100, B=200)(f)
+        ok (d) == {'A': 10, 'B': 20, 'C': 30 }
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_values2 ... [ok]\n"
+do_test_with(desc, script, expected)
+
+### dummy_attrs (with with-statement)
+desc = "dummy_attrs (with with-statement)"
+script = r"""
+from __future__ import with_statement
+import os
+from oktest import *
+class Foo(object):
+  def __init__(self, x, y):
+    self.x = x
+    self.y = y
+class FooTest(object):
+    def test_dummy_attrs(self):
+        obj = Foo(10, 20)
+        with dummy_attrs(obj, x=100, z=300):
+          ok (obj.x) == 100
+          ok (obj.y) == 20
+          ok (obj.z) == 300
+        #
+        ok (obj.x) == 10
+        ok (obj.y) == 20
+        ok (hasattr(obj, 'z')) == False
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_attrs ... [ok]\n"
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+### dummy_attrs (without with-statement)
+desc = "dummy_attrs (without with-statement)"
+script = r"""
+import os
+from oktest import *
+class Foo(object):
+  def __init__(self, x, y):
+    self.x = x
+    self.y = y
+class FooTest(object):
+    def test_dummy_attrs2(self):
+        obj = Foo(10, 20)
+        def f():
+          ok (obj.x) == 100
+          ok (obj.y) == 20
+          ok (obj.z) == 300
+        dummy_attrs(obj, x=100, z=300)(f)
+        #
+        ok (obj.x) == 10
+        ok (obj.y) == 20
+        ok (hasattr(obj, 'z')) == False
+run(FooTest)
+"""
+expected = "* FooTest.test_dummy_attrs2 ... [ok]\n"
+do_test_with(desc, script, expected)
+
 ### chdir (with with-statement)
 desc = "chdir (with with-statement)"
 script = r"""
@@ -705,6 +925,108 @@ run(FooTest)
 expected = "* FooTest.test_chdir2 ... [ok]\n"
 do_test_with(desc, script, expected)
 
+### spec (with with-statement)
+desc = "spec (with with-statement)"
+script = r"""
+from __future__ import with_statement
+from oktest import *
+class FooTest(object):
+    def test_spec1(self):
+        with spec('1+1 is 2') as sp:
+            ok (1+1) == 2
+        ok (sp.desc) == '1+1 is 2'
+    def test_spec2(self):
+        with spec('1-1 is 0') as sp:
+            ok (1-1) == -1
+run(FooTest)
+"""
+expected = """
+* FooTest.test_spec1 ... [ok]
+* FooTest.test_spec2 ... [NG] 0 == -1 : failed.
+   _test_.py:11: ok (1-1) == -1
+"""[1:]
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+### using (with with-statement)
+desc = "using (with with-statement)"
+script = r"""
+from __future__ import with_statement
+from oktest import *
+class FooTest(object):
+    pass
+with using(FooTest):
+    def test_1(self):
+        ok (1+1) == 2
+    def test_2(self):
+        ok (1*1) == 1
+run(FooTest)
+"""
+expected = """
+* FooTest.test_1 ... [ok]
+* FooTest.test_2 ... [ok]
+"""[1:]
+if with_statement_supported:
+    do_test_with(desc, script, expected)
+
+
+## flatten
+desc = "flatten()"
+script = r"""
+from oktest import ok, run, flatten
+class FooTest(object):
+    def test_flatten(self):
+        ok (flatten([1, [2, 3, [4, 5, [[[6]]]], [7, 8]]])) == [1,2,3,4,5,6,7,8]
+run(FooTest)
+"""
+expected = """
+* FooTest.test_flatten ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
+## rm_rf()
+desc = "rm_rf()"
+script = r"""
+import os
+from oktest import ok, not_ok, run, flatten, rm_rf
+class FooTest(object):
+    def setup(self):
+        os.mkdir('_rm_rf')
+        os.mkdir('_rm_rf/A')
+        os.mkdir('_rm_rf/B')
+        os.mkdir('_rm_rf/B/C')
+        open('_rm_rf/B/C/X.txt', 'w').write('xxx')
+        open('_rm_rf/Y.txt', 'w').write('yyy')
+        assert os.path.isfile('_rm_rf/B/C/X.txt')
+        assert os.path.isfile('_rm_rf/Y.txt')
+    def after(setup):
+        import shutil
+        if os.path.isdir('_rm_rf'):
+            shutil.rmtree('_rm_rf')
+    def test_remove_files_recursively(self):
+        args = ['_rm_rf/A', '_rm_rf/B', '_rm_rf/Y.txt']
+        rm_rf(*args)
+        for arg in flatten(args):
+            ok (os.path.exists(arg)) == False
+    def test_flatten_args(self):
+        args = ['_rm_rf/A', ['_rm_rf/B', '_rm_rf/Y.txt']]
+        rm_rf(args)
+        for arg in flatten(args):
+            ok (os.path.exists(arg)) == False
+    def test_ignore_unexist_files(self):
+        args = ['_rm_rf/A', '_rm_rf/K', '_rm_rf/Z.txt']
+        def f():
+            rm_rf(*args)
+        not_ok (f).raises(Exception)
+run(FooTest)
+"""
+expected = """
+* FooTest.test_remove_files_recursively ... [ok]
+* FooTest.test_flatten_args ... [ok]
+* FooTest.test_ignore_unexist_files ... [ok]
+"""[1:]
+do_test_with(desc, script, expected)
+
 
 ### diff (oktest.DIFF = True)
 desc = "diff (oktest.DIFF = True)"
@@ -869,3 +1191,29 @@ ERROR: FooTest#test_error()
 """[1:]
 os.environ['OKTEST_REPORTER'] = 'SimpleReporter'
 do_test_with(desc, script, expected)
+del os.environ['OKTEST_REPORTER']
+
+
+## checking tested or not
+desc = "checking tested or not"
+script = r"""
+from oktest import *
+import sys
+sys.stderr = sys.stdout
+import unittest
+class FooTest(object):
+  def test_1(self):
+    ok (1+1) == 2
+  def test_2(self):
+    ok (1+1)
+  def test_3(self):
+    not_ok (1+1)
+run(FooTest)
+"""[1:]
+expected = r"""
+### FooTest
+.*** warning: oktest: ok() is called but not tested. (file '_test_.py', line 9)
+.*** warning: oktest: not_ok() is called but not tested. (file '_test_.py', line 11)
+.
+"""[1:]
+do_test_with(desc, script, expected)
